@@ -650,9 +650,8 @@ ProcShmGetImage(ClientPtr client)
                wBorderWidth((WindowPtr) pDraw) + (int) pDraw->height)
             return BadMatch;
         visual = wVisual(((WindowPtr) pDraw));
-        pVisibleRegion = NotClippedByChildren((WindowPtr) pDraw);
-        if (pVisibleRegion)
-            RegionTranslate(pVisibleRegion, -pDraw->x, -pDraw->y);
+        if (pDraw->type == DRAWABLE_WINDOW)
+            pVisibleRegion = &((WindowPtr) pDraw)->borderClip;
     }
     else {
         if (stuff->x < 0 ||
@@ -715,9 +714,6 @@ ProcShmGetImage(ClientPtr client)
         }
     }
 
-    if (pVisibleRegion)
-        RegionDestroy(pVisibleRegion);
-
     if (client->swapped) {
         swaps(&xgi.sequenceNumber);
         swapl(&xgi.length);
@@ -991,7 +987,7 @@ ProcPanoramiXShmCreatePixmap(ClientPtr client)
                               RT_PIXMAP, pMap, RT_NONE, NULL, DixCreateAccess);
             if (result != Success) {
                 pDraw->pScreen->DestroyPixmap(pMap);
-                return result;
+                break;
             }
             dixSetPrivate(&pMap->devPrivates, shmPixmapPrivateKey, shmdesc);
             shmdesc->refcnt++;
@@ -1008,7 +1004,7 @@ ProcPanoramiXShmCreatePixmap(ClientPtr client)
         }
     }
 
-    if (result == BadAlloc) {
+    if (result != Success) {
         while (j--)
             FreeResource(newPix->info[j].id, RT_NONE);
         free(newPix);
@@ -1199,7 +1195,6 @@ shm_tmpfile(void)
 {
 #ifdef SHMDIR
 	int	fd;
-	int	flags;
 	char	template[] = SHMDIR "/shmfd-XXXXXX";
 #ifdef O_TMPFILE
 	fd = open(SHMDIR, O_TMPFILE|O_RDWR|O_CLOEXEC|O_EXCL, 0666);
@@ -1209,15 +1204,21 @@ shm_tmpfile(void)
 	}
 	ErrorF ("Not using O_TMPFILE\n");
 #endif
+#ifdef HAVE_MKOSTEMP
+	fd = mkostemp(template, O_CLOEXEC);
+#else
 	fd = mkstemp(template);
+#endif
 	if (fd < 0)
 		return -1;
 	unlink(template);
-	flags = fcntl(fd, F_GETFD);
+#ifndef HAVE_MKOSTEMP
+	int flags = fcntl(fd, F_GETFD);
 	if (flags != -1) {
 		flags |= FD_CLOEXEC;
 		(void) fcntl(fd, F_SETFD, &flags);
 	}
+#endif
 	return fd;
 #else
         return -1;
@@ -1238,6 +1239,7 @@ ProcShmCreateSegment(ClientPtr client)
     };
 
     REQUEST_SIZE_MATCH(xShmCreateSegmentReq);
+    LEGAL_NEW_RESOURCE(stuff->shmseg, client);
     if ((stuff->readOnly != xTrue) && (stuff->readOnly != xFalse)) {
         client->errorValue = stuff->readOnly;
         return BadValue;
@@ -1336,7 +1338,7 @@ ProcShmDispatch(ClientPtr client)
     }
 }
 
-static void
+static void _X_COLD
 SShmCompletionEvent(xShmCompletionEvent * from, xShmCompletionEvent * to)
 {
     to->type = from->type;
@@ -1348,7 +1350,7 @@ SShmCompletionEvent(xShmCompletionEvent * from, xShmCompletionEvent * to)
     cpswapl(from->offset, to->offset);
 }
 
-static int
+static int _X_COLD
 SProcShmQueryVersion(ClientPtr client)
 {
     REQUEST(xShmQueryVersionReq);
@@ -1357,7 +1359,7 @@ SProcShmQueryVersion(ClientPtr client)
     return ProcShmQueryVersion(client);
 }
 
-static int
+static int _X_COLD
 SProcShmAttach(ClientPtr client)
 {
     REQUEST(xShmAttachReq);
@@ -1368,7 +1370,7 @@ SProcShmAttach(ClientPtr client)
     return ProcShmAttach(client);
 }
 
-static int
+static int _X_COLD
 SProcShmDetach(ClientPtr client)
 {
     REQUEST(xShmDetachReq);
@@ -1378,7 +1380,7 @@ SProcShmDetach(ClientPtr client)
     return ProcShmDetach(client);
 }
 
-static int
+static int _X_COLD
 SProcShmPutImage(ClientPtr client)
 {
     REQUEST(xShmPutImageReq);
@@ -1399,7 +1401,7 @@ SProcShmPutImage(ClientPtr client)
     return ProcShmPutImage(client);
 }
 
-static int
+static int _X_COLD
 SProcShmGetImage(ClientPtr client)
 {
     REQUEST(xShmGetImageReq);
@@ -1416,7 +1418,7 @@ SProcShmGetImage(ClientPtr client)
     return ProcShmGetImage(client);
 }
 
-static int
+static int _X_COLD
 SProcShmCreatePixmap(ClientPtr client)
 {
     REQUEST(xShmCreatePixmapReq);
@@ -1432,7 +1434,7 @@ SProcShmCreatePixmap(ClientPtr client)
 }
 
 #ifdef SHM_FD_PASSING
-static int
+static int _X_COLD
 SProcShmAttachFd(ClientPtr client)
 {
     REQUEST(xShmAttachFdReq);
@@ -1443,7 +1445,7 @@ SProcShmAttachFd(ClientPtr client)
     return ProcShmAttachFd(client);
 }
 
-static int
+static int _X_COLD
 SProcShmCreateSegment(ClientPtr client)
 {
     REQUEST(xShmCreateSegmentReq);
@@ -1455,7 +1457,7 @@ SProcShmCreateSegment(ClientPtr client)
 }
 #endif  /* SHM_FD_PASSING */
 
-static int
+static int _X_COLD
 SProcShmDispatch(ClientPtr client)
 {
     REQUEST(xReq);
