@@ -112,11 +112,10 @@ printModeRejectMessage(int index, DisplayModePtr p, int status)
 }
 
 /*
- * xf86GetNearestClock --
- *	Find closest clock to given frequency (in kHz).  This assumes the
- *	number of clocks is greater than zero.
+ * Find closest clock to given frequency (in kHz).  This assumes the
+ * number of clocks is greater than zero.
  */
-int
+static int
 xf86GetNearestClock(ScrnInfoPtr scrp, int freq, Bool allowDiv2,
                     int DivFactor, int MulFactor, int *divider)
 {
@@ -451,7 +450,7 @@ xf86HandleBuiltinMode(ScrnInfoPtr scrp,
  * reason.
  */
 
-ModeStatus
+static ModeStatus
 xf86LookupMode(ScrnInfoPtr scrp, DisplayModePtr modep,
                ClockRangePtr clockRanges, LookupModeFlags strategy)
 {
@@ -478,7 +477,7 @@ xf86LookupMode(ScrnInfoPtr scrp, DisplayModePtr modep,
         M_T_DRIVER,
         0
     };
-    const int ntypes = sizeof(types) / sizeof(int);
+    const int ntypes = ARRAY_SIZE(types);
 
     strategy &= ~(LOOKUP_CLKDIV2 | LOOKUP_OPTIONAL_TOLERANCES);
 
@@ -841,11 +840,9 @@ xf86CheckModeSize(ScrnInfoPtr scrp, int w, int x, int y)
  *    monitor      pointer to structure for monitor section
  *    fbFormat     pixel format for the framebuffer
  *    videoRam     video memory size (in kB)
- *    maxHValue    maximum horizontal timing value
- *    maxVValue    maximum vertical timing value
  */
 
-ModeStatus
+static ModeStatus
 xf86InitialCheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode,
                               ClockRangePtr clockRanges,
                               LookupModeFlags strategy,
@@ -889,12 +886,6 @@ xf86InitialCheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode,
     if (virtualY > 0 && mode->VDisplay > virtualY)
         return MODE_VIRTUAL_Y;
 
-    if (scrp->maxHValue > 0 && mode->HTotal > scrp->maxHValue)
-        return MODE_BAD_HVALUE;
-
-    if (scrp->maxVValue > 0 && mode->VTotal > scrp->maxVValue)
-        return MODE_BAD_VVALUE;
-
     /*
      * The use of the DisplayModeRec's Crtc* and SynthClock elements below is
      * provisional, in that they are later reused by the driver at mode-set
@@ -986,8 +977,6 @@ xf86InitialCheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode,
  *    flags        not (currently) used
  *
  * In addition, the following fields from the ScrnInfoRec are used:
- *    maxHValue    maximum horizontal timing value
- *    maxVValue    maximum vertical timing value
  *    virtualX     virtual width
  *    virtualY     virtual height
  *    clockRanges  allowable clock ranges
@@ -1022,12 +1011,6 @@ xf86CheckModeForDriver(ScrnInfoPtr scrp, DisplayModePtr mode, int flags)
     if (mode->VDisplay > scrp->virtualY)
         return MODE_VIRTUAL_Y;
 
-    if (scrp->maxHValue > 0 && mode->HTotal > scrp->maxHValue)
-        return MODE_BAD_HVALUE;
-
-    if (scrp->maxVValue > 0 && mode->VTotal > scrp->maxVValue)
-        return MODE_BAD_VVALUE;
-
     for (cp = scrp->clockRanges; cp != NULL; cp = cp->next) {
         /* DivFactor and MulFactor must be > 0 */
         cp->ClockDivFactor = max(1, cp->ClockDivFactor);
@@ -1334,8 +1317,6 @@ scanLineWidth(unsigned int xsize,       /* pixels */
  *    monitor      pointer to structure for monitor section
  *    fbFormat     format of the framebuffer
  *    videoRam     video memory size
- *    maxHValue    maximum horizontal timing value
- *    maxVValue    maximum vertical timing value
  *    xInc         horizontal timing increment (defaults to 8 pixels)
  *
  * The function fills in the following ScrnInfoRec fields:
@@ -1527,8 +1508,6 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
         }
     }
 
-    /* Initial check of virtual size against other constraints */
-    scrp->virtualFrom = X_PROBED;
     /*
      * Initialise virtX and virtY if the values are fixed.
      */
@@ -1580,7 +1559,6 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
 
         virtX = virtualX;
         virtY = virtualY;
-        scrp->virtualFrom = X_CONFIG;
     }
     else if (!modeNames || !*modeNames) {
         /* No virtual size given in the config, try to infer */
@@ -1665,12 +1643,8 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
     }
 
     /* Lookup each mode */
-#ifdef RANDR
-    if (!xf86Info.disableRandR
 #ifdef PANORAMIX
-        && noPanoramiXExtension
-#endif
-        )
+    if (noPanoramiXExtension)
         validateAllDefaultModes = TRUE;
 #endif
 
@@ -1875,7 +1849,7 @@ xf86ValidateModes(ScrnInfoPtr scrp, DisplayModePtr availModes,
                 M_T_DRIVER,
                 0
             };
-            const int ntypes = sizeof(types) / sizeof(int);
+            const int ntypes = ARRAY_SIZE(types);
             int n;
 
             /*
@@ -2086,9 +2060,8 @@ xf86PrintModes(ScrnInfoPtr scrp)
     if (scrp == NULL)
         return;
 
-    xf86DrvMsg(scrp->scrnIndex, scrp->virtualFrom, "Virtual size is %dx%d "
-               "(pitch %d)\n", scrp->virtualX, scrp->virtualY,
-               scrp->displayWidth);
+    xf86DrvMsg(scrp->scrnIndex, X_INFO, "Virtual size is %dx%d (pitch %d)\n",
+               scrp->virtualX, scrp->virtualY, scrp->displayWidth);
 
     p = scrp->modes;
     if (p == NULL)
