@@ -100,10 +100,9 @@ typedef struct _Client {
     CARD32 req_len;             /* length of current request */
     unsigned int replyBytesRemaining;
     PrivateRec *devPrivates;
-    unsigned short xkbClientFlags;
     unsigned short mapNotifyMask;
     unsigned short newKeyboardNotifyMask;
-    unsigned short vMajor, vMinor;
+    unsigned char xkbClientFlags;
     KeyCode minKC, maxKC;
 
     int smart_start_tick;
@@ -111,9 +110,7 @@ typedef struct _Client {
 
     DeviceIntPtr clientPtr;
     ClientIdPtr clientIds;
-#if XTRANS_SEND_FDS
     int req_fds;
-#endif
 } ClientRec;
 
 #if XTRANS_SEND_FDS
@@ -132,7 +129,7 @@ extern long SmartScheduleTime;
 extern long SmartScheduleInterval;
 extern long SmartScheduleSlice;
 extern long SmartScheduleMaxSlice;
-#if HAVE_SETITIMER
+#ifdef HAVE_SETITIMER
 extern Bool SmartScheduleSignalEnable;
 #else
 #define SmartScheduleSignalEnable FALSE
@@ -143,6 +140,12 @@ extern void SmartScheduleStopTimer(void);
 /* Client has requests queued or data on the network */
 void mark_client_ready(ClientPtr client);
 
+/*
+ * Client has requests queued or data on the network, but awaits a
+ * server grab release
+ */
+void mark_client_saved_ready(ClientPtr client);
+
 /* Client has no requests queued and no data on network */
 void mark_client_not_ready(ClientPtr client);
 
