@@ -117,18 +117,6 @@ present_flip_pending_pixmap(ScreenPtr screen)
     return screen_priv->flip_pending->pixmap;
 }
 
-static Bool
-present_check_output_slaves_active(ScreenPtr pScreen)
-{
-    ScreenPtr pSlave;
-
-    xorg_list_for_each_entry(pSlave, &pScreen->slave_list, slave_head) {
-        if (RRHasScanoutPixmap(pSlave))
-            return TRUE;
-    }
-    return FALSE;
-}
-
 static Bool
 present_check_flip(RRCrtcPtr    crtc,
                    WindowPtr    window,
@@ -156,10 +144,6 @@ present_check_flip(RRCrtcPtr    crtc,
     if (!screen_priv->info->flip)
         return FALSE;
 
-    /* Fail to flip if we have slave outputs */
-    if (screen->output_slaves && present_check_output_slaves_active(screen))
-        return FALSE;
-
     /* Make sure the window hasn't been redirected with Composite */
     window_pixmap = screen->GetWindowPixmap(window);
     if (window_pixmap != screen->GetScreenPixmap(screen) &&
@@ -543,8 +527,6 @@ present_event_notify(uint64_t event_id, uint64_t ust, uint64_t msc)
             present_execute(vblank, ust, msc);
             return;
         }
-        if (match < 0)
-            break;
     }
     xorg_list_for_each_entry(vblank, &present_flip_queue, event_queue) {
         if (vblank->event_id == event_id) {
@@ -1007,8 +989,6 @@ present_abort_vblank(ScreenPtr screen, RRCrtcPtr crtc, uint64_t event_id, uint64
             vblank->queued = FALSE;
             return;
         }
-        if (match < 0)
-            break;
     }
     xorg_list_for_each_entry(vblank, &present_flip_queue, event_queue) {
         if (vblank->event_id == event_id) {
